<?php

namespace App\Http\Controllers;

use Illuminate\Foundation\Auth\Access\AuthorizesRequests;
use Illuminate\Foundation\Bus\DispatchesJobs;
use Illuminate\Foundation\Validation\ValidatesRequests;
use Illuminate\Routing\Controller as BaseController;

// Controladores importados
use App\Http\Controllers\ViewsAdminController;
use App\Http\Controllers\HelpersAdminController;


// Mains
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Gate;
use Validator;
use Illuminate\Support\Str;



// Utils
use Illuminate\Support\Facades\DB;
use DateTime;
use DateInterval;
use DatePeriod;
use Carbon\Carbon;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Arr;
use Illuminate\Validation\Rules\Password;
use File;


// Models
use App\Models\Configuracion;
use App\Models\CategoriaInstalaciones;
use App\Models\CategoriaNoticias;
use App\Models\Instalacion;
use App\Models\TurnoDefault;
use App\Models\TurnoPreciso;
use App\Models\SubdivisionPrecisa;
use App\Models\SubdivisionDefault;
use App\Models\TipoSub;
use App\Models\Reserva;
use App\Models\User;
use App\Models\Asociado;
use App\Models\CategoriaAsociado;
use App\Models\Slider;

use App\Email;
use Mail;

class DataAdminController extends BaseController
{
    use AuthorizesRequests, DispatchesJobs, ValidatesRequests;


    /*
      **************************************************
        Importacion de controladores
      **************************************************
    */

    // protected $WSController;
    //
    // public function __construct(WSController $WSController)
    // {
    //     $this->WSController = $WSController;
    // }



    /*
      **************************************************
        Funciones de CRUD
      **************************************************
    */

    // INSTALACIONES
    // *****************************************

    // SUBDIVISIONES DEFAULT
    // *****************************************

    public function delete_subdivision_def(Request $request) {
      // TODO::AGREGAR COMPROBACIONES
      $sub = SubdivisionDefault::where('id', '=', $request['id'])->first();
      $sub->delete();
    }

    public function save_subdivision_def(Request $request) {
      // TODO::AGREGAR COMPROBACIONES
      $sub = SubdivisionDefault::where('id', '=', $request['id'])->first();

      $sub->descripcion = $request['nombre'];
      $sub->hora_apertura = $request['inicio'];
      $sub->hora_cierre = $request['final'];
      $sub->plazas_max = $request['plazas'];
      $sub->id_tipo = $request['tipo_id'];

      $sub->save();
    }

    public function add_subdivision_def(Request $request) {
      // TODO::AGREGAR COMPROBACIONES
      $sub = new SubdivisionDefault;

      $sub->descripcion = $request['nombre'];
      $sub->hora_apertura = $request['inicio'];
      $sub->hora_cierre = $request['final'];
      $sub->plazas_max = $request['plazas'];
      $sub->id_tipo = $request['tipo_id'];
      $sub->id_ins = $request['id_ins'];

      $sub->save();
    }

    // SUBDIVISIONES PRECISAS
    // *****************************************

    public function delete_subdivision_pre(Request $request) {
      // TODO::AGREGAR COMPROBACIONES
      $sub = SubdivisionPrecisa::where('id', '=', $request['id'])->first();
      $sub->delete();
    }

    public function save_subdivision_pre(Request $request) {
      // dd($request['dia']);
      // TODO::AGREGAR COMPROBACIONES
      $sub = SubdivisionPrecisa::where('id', '=', $request['id'])->first();

      $sub->descripcion = $request['nombre'];
      $sub->hora_apertura = $request['inicio'];
      $sub->hora_cierre = $request['final'];
      $sub->plazas_max = $request['plazas'];
      $sub->id_tipo = $request['tipo_id'];

      if($request['dia'] != null) :
        $sub->dia = $request['dia'];
      else :
        $sub->dia = NULL;
      endif;

      if($request['dia_semana'] != null && $request['dia_semana'] != 0) :
        $sub->dia_semana = $request['dia_semana'];
      else :
        $sub->dia_semana = 0;
      endif;

      $sub->save();
    }

    public function add_subdivision_pre(Request $request) {
      // TODO::AGREGAR COMPROBACIONES
      $sub = new SubdivisionPrecisa;

      // dd($request);

      $sub->descripcion = $request['nombre'];
      $sub->hora_apertura = $request['inicio'];
      $sub->hora_cierre = $request['final'];
      $sub->plazas_max = $request['plazas'];
      $sub->id_tipo = $request['tipo_id'];
      $sub->id_ins = $request['id_ins'];

      if($request['dia'] != null) :
        $sub->dia = $request['dia'];
      else :
        $sub->dia = NULL;
      endif;

      if($request['dia_semana'] != null && $request['dia_semana'] != 0) :
        $sub->dia_semana = $request['dia_semana'];
      else :
        $sub->dia_semana = 0;
      endif;

      $sub->save();
    }

    public function cambiar_estado_div(Request $request) {
      if($request['tipo'] == 'def') :
        $sub = SubdivisionDefault::where('id', '=', $request['id'])->first();
        $sub->change_estado();
        $pestana = 2;
      else :
        $sub = SubdivisionPrecisa::where('id', '=', $request['id'])->first();
        $sub->change_estado();
        $pestana = 3;
      endif;

      $sub->save();
      session([
        'pestana_actual' => $pestana
      ]);

      return response()->json([
        'estado' => $sub->get_estado()
      ]);
    }

    public function cambiar_estado_cat_instalaciones(Request $request) {
        $cat = CategoriaInstalaciones::where('id', '=', $request['id'])->first();
        $cat->change_estado();

      $cat->save();

      return response()->json([
        'estado' => $cat->get_estado()
      ]);
    }

    public function save_cat_instalaciones(Request $request) {
      if(isset($request['id'])) {
        $cat = CategoriaInstalaciones::where('id', '=', $request['id'])->first();
      } else {
        $cat = new CategoriaInstalaciones();
      }

        $cat->nombre = $request['nombre'];
        $cat->descripcion = $request['desc'];
        $cat->activo = $request['activo'];
        $cat->imagen = '';

      $saved = $cat->save();

      return response()->json([
        'success' => true,
        'id' => $cat->id
      ]);
    }

    public function save_cat_noticias(Request $request) {
      if(isset($request['id'])) {
        $cat = CategoriaNoticias::where('id', '=', $request['id'])->first();
      } else {
        $cat = new CategoriaNoticias();
      }

        $cat->nombre = $request['nombre'];
        $cat->descripcion = $request['desc'];
        $cat->activo = $request['activo'];
        $cat->imagen = '';

      $saved = $cat->save();

      return response()->json([
        'success' => true,
        'id' => $cat->id
      ]);
    }

    // TIPOS DE SUBDIVISIONES
    // *****************************************

    public function delete_tipo_subdivision(Request $request) {
      // TODO::AGREGAR COMPROBACIONES
      $tipo = TipoSub::where('id', '=', $request['id'])->first();
      $tipo->delete();
    }

    public function add_tipo_subdivision(Request $request) {
      // TODO::AGREGAR COMPROBACIONES
      $tipo = new TipoSub;

      // dd($request);

      $tipo->id_ins = $request['id_ins'];
      $tipo->nombre = $request['nombre'];
      $tipo->descripcion = $request['descripcion'];

      if($request['activo'] != false) :
        $tipo->activo = 1;
      else :
        $tipo->activo = 0;
      endif;

      $tipo->save();
    }


    // INSTALACIONES
    // *****************************************

    public function save_instalacion(Request $request) {
      // TODO::AGREGAR COMPROBACIONES
      // dd($request);
      if(isset($request['id'])) :
        $ins = Instalacion::where('id', '=', $request['id'])->first();
      else :
        $ins = new Instalacion();
      endif;

      $ins->nombre = $request['nombre'];
      $ins->hora_apertura = $request['inicio'];
      $ins->hora_cierre = $request['final'];
      $ins->plazas_max = $request['plazas'];
      $ins->id_categoria = $request['categoria'];
      $ins->descripcion = $request['desc'];
      $ins->descripcion_corta = $request['desc_corta'];
      $ins->activo = $request['activo'];
      $ins->reservable = $request['reservable'];
      $ins->reservas_multiples = $request['multiples'];
      $ins->subdivisible = $request['divisible'];
      $ins->nombre_subdivisiones = $request['nombre_sub'];
      $ins->lim_res_persona = $request['lim_persona'];
      $ins->lim_res_dia = $request['lim_dia'];
      $ins->h_apertura_reserva = $request['horas_apertura'];
      $ins->tiempo_previo = $request['minutos_previos'];
      $ins->mensaje_bienvenida = $request['mens_bien'];
      $ins->mensaje_reserva = $request['mens_reserva'];
      $ins->contacto_nombre = $request['nombre_contacto'];
      $ins->contacto_telefono = $request['tel_contacto'];
      $ins->link_entrada = $request['link_entrada'];

      if($ins->save()) :
        return response()->json([
            'success' => true,
            'id' => $ins->id
        ]);
      else :
        return response()->json([
            'success' => false
        ]);
      endif;
    }
    // ASOCIADOS
    // *****************************************

    public function save_asociado(Request $request) {
      // TODO::AGREGAR COMPROBACIONES
      // dd($request);
      $data = $request['data'];
      if(isset($request['id']) && !empty($request['id'])) :
        $asoc = Asociado::where('id', '=', $request['id'])->first();
      else :
        $asoc = new Asociado();
      endif;

      $asoc->nombre = $data['nombre'];
      $asoc->email = $data['email'];
      $asoc->direccion = $data['direccion'];
      $asoc->telefono = $data['telefono'];
      $asoc->web_link = $data['web_link'];
      $asoc->activo = $data['activo'];
      $asoc->destacado = $data['destacado'];
      $asoc->id_categorias = json_encode($data['categorias']);
      $asoc->texto_corto = $data['texto_corto'];
      $asoc->texto = $data['texto'];

      if($asoc->save()) :
        return response()->json([
            'success' => true,
            'id' => $asoc->id
        ]);
      else :
        return response()->json([
            'success' => false
        ]);
      endif;
    }

    public function save_categoria_asociado(Request $request) {
      // TODO::AGREGAR COMPROBACIONES
      // dd($request);
      $data = $request['data'];
      if(isset($request['id']) && !empty($request['id'])) :
        $categoria = CategoriaAsociado::where('id', '=', $request['id'])->first();
      else :
        $categoria = new CategoriaAsociado();
      endif;

      $categoria->nombre = $data['nombre'];
      $categoria->activo = $data['activo'];
      $categoria->descripcion = $data['descripcion'];

      if($categoria->save()) :
        return response()->json([
            'success' => true,
            'id' => $categoria->id
        ]);
      else :
        return response()->json([
            'success' => false
        ]);
      endif;
    }

    // SLIDERS
    // *****************************************

    public function save_frame(Request $request) {
      // TODO::AGREGAR COMPROBACIONES
      $data = $request['data'];
      if(isset($request['id']) && !empty($request['id'])) :
        $slider = Slider::where('id', '=', $request['id'])->first();
      else :
        $slider = new Slider();
      endif;

     $slider->title = $data['title'];
     $slider->subtitle = $data['subtitle'];
     $slider->short_desc = html_entity_decode($data['short_desc'], ENT_QUOTES, 'UTF-8');
     if(isset($data['image'])) {
       $slider->image = $data['image'];
     }
     $slider->button_text = $data['button_text'];
     $slider->button_link = $data['button_link'];
     $slider->status = $data['status'];

      if($slider->save()) :
        return response()->json([
            'success' => true,
            'id' => $slider->id
        ]);
      else :
        return response()->json([
            'success' => false
        ]);
      endif;
    }


    public function set_vista_instalaciones(Request $request) {
      $vista = Configuracion::find(1)->set_vista('instalaciones', $request['value']);
    }



    /*
      **************************************************
        Funciones AJAX - JS - JQUERY
      **************************************************
    */

    // public function subir_archivo(Request $request) {
    //   $url = $request['url'];
    //   $my_file = $request->file('archivo');
    //   $file_name = trim($my_file->getClientOriginalName());

    //   switch($url) :
    //     case 'asociado' :
    //       $url_dir = Configuracion::find(1)->imagenes_upload_dir_asociados;
    //       $obj = Asociado::find($request['id_obj']);
    //       break;
    //     case 'categoria-asociado' :
    //       $url_dir = Configuracion::find(1)->imagenes_upload_dir_categorias_asociados;
    //       $obj = CategoriaAsociado::find($request['id_obj']);
    //       break;
    //     case 'slider' :
    //       $url_dir = Configuracion::find(1)->imagenes_upload_dir_slider_frames;
    //       $obj = Slider::find($request['id_obj']);
    //       break;
    //   endswitch;

    //   if(!File::exists(public_path($obj->get_imagen()))) :
    //     if($my_file->storeAs($url_dir, $file_name)) :
    //       $obj->del_imagen();
    //       // $obj->imagen = $my_file->getClientOriginalName();
    //       $obj->set_imagen($file_name);
    //       $obj->save();
    //     endif;
    //     return response()->json([
    //         'success' => true,
    //         'filename' => $file_name,
    //         'url' => $url_dir
    //     ]);
    //   else :
    //     return response()->json([
    //       'success' => false,
    //       'error' => "La imagen ya existe."
    //   ]);
    //   endif;
    // }

    public function subir_archivo(Request $request) {
      $url = $request->input('url');
      $my_file = $request->file('archivo');
      
    // Decodificar y sanitizar el nombre del archivo
    $originalFileName = urldecode($my_file->getClientOriginalName());
    $sanitizedFileName = $this->sanitizeFileName($originalFileName);
    
    // Obtener la extensión del archivo
    $extension = $my_file->getClientOriginalExtension();
    
    // Generar un nombre único para evitar colisiones
    $uniqueFileName = $this->generateUniqueFileName($sanitizedFileName, $extension);
  
      switch($url) {
          case 'asociado':
              $url_dir = Configuracion::find(1)->imagenes_upload_dir_asociados;
              $obj = Asociado::find($request->input('id_obj'));
              break;
          case 'categoria-asociado':
              $url_dir = Configuracion::find(1)->imagenes_upload_dir_categorias_asociados;
              $obj = CategoriaAsociado::find($request->input('id_obj'));
              break;
          case 'slider':
              $url_dir = Configuracion::find(1)->imagenes_upload_dir_slider_frames;
              $obj = Slider::find($request->input('id_obj'));
              break;
          default:
              return response()->json([
                  'success' => false,
                  'error' => 'URL no válida.'
              ]);
      }
  
      // Verificar si el archivo ya existe
      if (!File::exists(public_path($obj->get_imagen()))) {
          if ($my_file->storeAs($url_dir, $uniqueFileName)) {
              $obj->del_imagen();
              $obj->set_imagen($uniqueFileName);
              $obj->save();
  
              return response()->json([
                  'success' => true,
                  'filename' => $uniqueFileName,
                  'url' => $url_dir
              ]);
          }
      } else {
          return response()->json([
              'success' => false,
              'error' => 'La imagen ya existe.'
          ]);
      }
  }
  
    /**
     * Sanitiza el nombre del archivo reemplazando caracteres no válidos.
     *
     * @param string $fileName El nombre del archivo original.
     * @return string El nombre del archivo sanitizado.
     */
    private function sanitizeFileName($fileName) {
        // Reemplazar espacios y caracteres especiales por guiones bajos
        $fileName = preg_replace('/[^\w\-\.]/', '_', $fileName);
        // Quitar cualquier guión bajo múltiple o al final del nombre
        return preg_replace('/_+/', '_', trim($fileName, '_'));
    }

    /**
   * Genera un nombre único para el archivo añadiendo un identificador antes de la extensión.
   *
   * @param string $fileName El nombre del archivo sanitizado.
   * @param string $extension La extensión del archivo.
   * @return string El nombre del archivo único.
   */
  private function generateUniqueFileName($fileName, $extension) {
    // Eliminar la extensión original si está presente en el nombre
    $fileName = pathinfo($fileName, PATHINFO_FILENAME);
    // Añadir un identificador único antes de la extensión
    return $fileName . '_' . time() . '.' . $extension;
  }

    public function set_tipo_division(Request $request) {
      $reserva = Reserva::find($request['res_id']);

      // Datos subdivision
      $reserva->id_sub = $request['id_division'];
      $reserva->id_tipo_sub = $request['id_tipo_div'];
      $reserva->tipo_sub = $request['type'];
      // Datos turno
      if(!$request['turno_type']) :
        $turno = TurnoDefault::find($request['turno_id']);
      else :
        $turno = TurnoPreciso::find($request['turno_id']);
      endif;
      $reserva->tipo_turno = $request['turno_type'];
      $reserva->id_turno = $request['turno_id'];

      $reserva->hora_inicio = $turno->hora_comienzo;
      $reserva->hora_final = $turno->hora_termino;

      $fecha = new Carbon($request['fecha']);
      $reserva->fecha = $fecha->format('Y-m-d');

      $reserva->save();
    }

    public function guardar_admin_reserva(Request $request) {
      // TODO:: Filtro para que solo usuarios con permiso para crear reservas puedan acceder.

      if(isset($request['res_id'])) :
        $reserva = Reserva::find($request['res_id']);
      else :
        $reserva = new Reserva;
      endif;
      // Datos subdivision
      $reserva->id_sub = $request['id_division'];
      $reserva->id_tipo_sub = $request['id_tipo_div'];
      $reserva->tipo_sub = $request['type'];
      // Datos turno
      if(!$request['turno_type']) :
        $turno = TurnoDefault::find($request['turno_id']);
      else :
        $turno = TurnoPreciso::find($request['turno_id']);
      endif;
      $reserva->tipo_turno = $request['turno_type'];
      $reserva->id_turno = $request['turno_id'];

      $reserva->id_user = (int)$request['id_user'];
      $reserva->id_ins = $turno->instalacion->id;

      $reserva->hora_inicio = $turno->hora_comienzo;
      $reserva->hora_final = $turno->hora_termino;

      $fecha = new Carbon($request['fecha']);
      $reserva->fecha = $fecha->format('Y-m-d');

      if(!isset($request['res_id'])) :
        // Creamos la referencia de la booking
        $name4 =  Str::limit(Auth::user()->name, 4, '');
        $name4 = strtoupper($name4).Auth::user()->id;
        $random = Str::random(4);
        $fecha = new Carbon($reserva->fecha);
        $fecha_res = $fecha->clone()->format('d-m-Y');
        $fecha = $fecha->format('Mdy');
        $reserva->ref = 'HERAYTO-IN'.$turno->instalacion->id.'-'.$fecha.$turno->get_hora_comienzo().'-'.$name4;
      endif;

      $reserva->save();
    }


    /**
     *  USUARIO 
     */

    public function create_user(Request $request)
    {
      // dd($request);
      $new_user = new User();

      $new_user->name = $request['data']['user_name'];
      $new_user->email = $request['data']['user_email'];
      $new_user->telefono = $request['data']['user_phone'];
      $new_user->id_role = (int)$request['data']['user_role'];
      
      if($request['data']['first']) :
        $new_user->cambiar_password = 1;
        $new_user->password = '';
      else :
        $new_user->cambiar_password = 0;
        $valid_pass = self::InternalValidationPassword($request['data']);
        $valid_pass->success ? $new_user->pass = $request['data']['pass'] : '';
      endif;

      
      if(!$new_user->save()) :
        return response()->json([
          'success' => false   
        ]);
      else :
        return response()->json([
          'success' => true   
        ]);
      endif;
    }

    public function save_user(Request $request)
    {
      dd($request);
      $new_user = new User();

      $new_user->name = $request['name'];
    }

    public function bloquear_usuario(Request $request) {
      $user = User::where('id', '=', $request['id'])->first();
      $user->id_status = 3;
      $user->save();

      return response()->json([
        'success' => true
      ]);
    }

    public function pausar_usuario(Request $request) {
      $user = User::where('id', '=', $request['id'])->first();
      $user->id_status = 2;
      $user->save();

      return response()->json([
        'success' => true
      ]);
    }

    public function desbloquear_usuario(Request $request) {
      $user = User::where('id', '=', $request['id'])->first();
      $user->id_status = 1;
      $user->save();

      return response()->json([
        'success' => true
      ]);
    }

    public function bloquear_asociado(Request $request) {
      $asociado = Asociado::where('id', '=', $request['id'])->first();
      $asociado->activo = 0;
      $asociado->save();

      return response()->json([
        'success' => true
      ]);
    }

    public function pausar_asociado(Request $request) {
      $asociado = Asociado::where('id', '=', $request['id'])->first();
      $asociado->activo = 0;
      $asociado->save();

      return response()->json([
        'success' => true
      ]);
    }

    public function desbloquear_asociado(Request $request) {
      $asociado = Asociado::where('id', '=', $request['id'])->first();
      $asociado->activo = 1;
      $asociado->save();

      return response()->json([
        'success' => true
      ]);
    }
    public function bloquear_frame(Request $request) {
      $slider = Slider::where('id', '=', $request['id'])->first();
      $slider->status = 0;
      $slider->save();

      return response()->json([
        'success' => true
      ]);
    }

    public function pausar_frame(Request $request) {
      $slider = Slider::where('id', '=', $request['id'])->first();
      $slider->status = 0;
      $slider->save();

      return response()->json([
        'success' => true
      ]);
    }

    public function desbloquear_frame(Request $request) {
      $slider = Slider::where('id', '=', $request['id'])->first();
      $slider->status = 1;
      $slider->save();

      return response()->json([
        'success' => true
      ]);
    }


    /*
      **************************************************
        HELPERS
      **************************************************
    */


        public function InternalValidationPassword(Request $request) {
          $validator = Validator::make($request->all(),[
            'pass' => [
              'required',
              Password::min(8)
              ->mixedCase() //Al menos una mayuscula y una minuscula
              ->symbols() //Al menus un simbolo
              ->uncompromised() //Sin comprometer para pwnedpasswords.com (dataleak)
            ],
            'pass_confirmation' => 'same:pass'
          ]);

          if ($validator->fails()){
            $errors = $validator->errors();
            return (Object)[
              'success' => false,
              'myerrors' => $errors
            ];
          } else {
            return (object)[
              'success' => true
            ];
          }
        }

        // Función que comprueba si un usuario existe y si está en estado de cambio de password o no.
        public function comprobarUserStatus(Request $request) {
          $user = Usuario::where('VCODUSU', '=', $request['user'])->where('VCAMBIAR_PASS_PROXIMO_ACCESO', '=', 'S')->first();
          $response = (!$user == NULL);

          return response()->json([
              'have_to_change' => $response
          ]);
        }

        public function saveOrder(Request $request)
        {
          $items = $request['data']['items'];

          foreach($items as $item) {
            $slider = Slider::where('id', '=', (int)$item['id'])->first();
            $slider->order = $item['order'];
            $saved = $slider->save();
          }

          if ($items == null || !$saved) {
            return response()->json([
              'success' => false,
              'myerrors' => "error"
            ]);
          } else {
            return response()->json([
              'success' => true,
              // 'items' => $
            ]);
          }
        }
}
