<?php

namespace App\Http\Controllers;

date_default_timezone_set('Europe/Madrid');

use Illuminate\Foundation\Auth\Access\AuthorizesRequests;
use Illuminate\Foundation\Bus\DispatchesJobs;
use Illuminate\Foundation\Validation\ValidatesRequests;
use Illuminate\Routing\Controller as BaseController;

// Controladores importados
use App\Http\Controllers\ViewsController;


// Mains
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Gate;
use Validator;
use Illuminate\Support\Str;


// Utils
use Illuminate\Support\Facades\DB;
use DateTime;
use DateInterval;
use DatePeriod;
use Carbon\Carbon;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Arr;
use Illuminate\Validation\Rules\Password;


// Models
use App\Models\Noticia;
use App\Models\Configuracion;
use App\Models\CategoriaInstalaciones;
use App\Models\Instalacion;
use App\Models\TurnoDefault;
use App\Models\TurnoInstalacion;
use App\Models\Reserva;


use App\Email;
use Mail;

class DataController extends BaseController
{
    use AuthorizesRequests, DispatchesJobs, ValidatesRequests;


    /*
      **************************************************
        Importacion de controladores
      **************************************************
    */

    // protected $WSController;
    //
    // public function __construct(WSController $WSController)
    // {
    //     $this->WSController = $WSController;
    // }



    /*
      **************************************************
        Funciones de CRUD
      **************************************************
    */

      public function save_reserva(Request $request) {
        // checqueamos que el usuario esté logueado.
        if(!Auth::check()) :
          $error = (object) [
            'code' => 666,
            'message_title' => 'Testing title',
            'message_text' => 'Probando el resultado mensaje a mostrar.'
          ];

          return response()->json([
            'error' => $error
          ]);

        else :

          if($request['tipo_turno'] == 'default') :
            $turno = TurnoDefault::find($request['id_turno']);
          else :
            $turno = TurnoPreciso::find($request['id_turno']);
          endif;

          if(HelpersController::user_have_turno($turno) || HelpersController::turno_full_booked($turno)) :
            $error = (object) [
              'code' => 666,
              'message_title' => 'Lugar ocupado',
              'message_text' => 'Lo sentimos alguien se le adelanto y ocupo el lugar antes que usted.'
            ];

            return response()->json([
              'error' => $error
            ]);
          endif;

          $instalacion = $turno->instalacion;

          $reserva = new Reserva;

          $reserva->id_user = Auth::user()->id;
          $reserva->id_ins = $instalacion->id;
          $reserva->id_turno = $turno->id;

          if($turno->TIPO == 'default') :
            $reserva->tipo_turno = 0;
          else :
            $reserva->tipo_turno = 1;
          endif;

          // Chequeamos si hay subdivisiones
          if($instalacion->has_subdivisiones()) :
            $reserva->id_sub  = $request['subdivision'];
            $tipo_sub = $request['tipo_sub_main'];
          endif;

          // TODO::Crear reservas multiples.
          $reserva->hora_inicio = $turno->hora_comienzo;
          $reserva->hora_final = $turno->hora_termino;
          $reserva->fecha = $turno->get_fecha_sql();

          // Creamos la referencia de la booking
          $name4 =  Str::limit(Auth::user()->name, 4, '');
          $name4 = strtoupper($name4).Auth::user()->id;
          $random = Str::random(4);
          $fecha = new Carbon($reserva->fecha);
          $fecha_res = $fecha->clone()->format('d-m-Y');
          $fecha = $fecha->format('Mdy');
          $reserva->ref = 'HERAYTO-IN'.$instalacion->id.'-'.$fecha.$turno->get_hora_comienzo().'-'.$name4;

          $reserva->save();

          $fecha_eje = Carbon::now();
          $fecha_eje = $fecha_eje->format('d-m-Y / H:m');

          $fecha_res = $fecha_res.' / '.$reserva->hora_inicio.' - '.$reserva->hora_final;
          // dd($fecha_back);

          return response()->json([
            'referencia' => $reserva->ref,
            'fecha_hora' => $fecha_eje,
            'fecha_res' => $fecha_res,
            'id_ins' => $instalacion->id,
            'tipo' => $reserva->id_tipo_sub,
            'tipo_sub' => $tipo_sub,
            'subdivision' => $reserva
          ]);

        endif;
      }

    /*
      **************************************************
        Funciones AJAX - JS - JQUERY
      **************************************************
    */



    /*
      **************************************************
        HELPERS
      **************************************************
    */

        public function InternalValidationPassword(Request $request) {

          $validator = Validator::make($request->all(),[
            'pass' => [
              'required',
              Password::min(8)
              ->mixedCase() //Al menos una mayuscula y una minuscula
              ->symbols() //Al menus un simbolo
              ->uncompromised() //Sin comprometer para pwnedpasswords.com (dataleak)
            ],
            'pass_confirmation' => 'same:pass'
          ]);

          if ($validator->fails()){
            $errors = $validator->errors();
            return (Object)[
              'success' => false,
              'myerrors' => $errors
            ];
          } else {
            return (object)[
              'success' => true
            ];
          }
        }


        // Función que comprueba si un usuario existe y si está en estado de cambio de password o no.
        public function comprobarUserStatus(Request $request) {
          $user = Usuario::where('VCODUSU', '=', $request['user'])->where('VCAMBIAR_PASS_PROXIMO_ACCESO', '=', 'S')->first();
          $response = (!$user == NULL);

          return response()->json([
              'have_to_change' => $response
          ]);
        }

        // Función que me devuelva las noticias a mostrar en la portada.
        // Máximo de 4 destacados solo.
        public function getNoticiasPortada() {
          // Recojo y devuelvo las 4 noticias destacadas.
          $noticias_portada = Noticia::Destacadas()->get();
          return $noticias_portada;

        }





}
