<?php

namespace App\Http\Controllers;

date_default_timezone_set('Europe/Madrid');


use Illuminate\Foundation\Auth\Access\AuthorizesRequests;
use Illuminate\Foundation\Bus\DispatchesJobs;
use Illuminate\Foundation\Validation\ValidatesRequests;
use Illuminate\Routing\Controller as BaseController;

// Controladores importados
use App\Http\Controllers\ViewsController;


// Mains
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Gate;
use Validator;


// Utils
use Illuminate\Support\Facades\DB;
use DateTime;
use DateInterval;
use DatePeriod;
use Carbon\Carbon;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Arr;
use Illuminate\Validation\Rules\Password;


// Models
use App\Models\Noticia;
use App\Models\Configuracion;
use App\Models\CategoriaInstalaciones;
use App\Models\Instalacion;
use App\Models\TurnoDefault;
use App\Models\TurnoInstalacion;
use App\Models\Reserva;

use App\Email;
use Mail;

class HelpersController extends BaseController
{
    use AuthorizesRequests, DispatchesJobs, ValidatesRequests;


    /*
      **************************************************
        Importacion de controladores
      **************************************************
    */

    // protected $WSController;
    //
    // public function __construct(WSController $WSController)
    // {
    //     $this->WSController = $WSController;
    // }



    /*
      **************************************************
        Funciones de CRUD
      **************************************************
    */



    /*
      **************************************************
        Funciones AJAX - JS - JQUERY
      **************************************************
    */



    /*
      **************************************************
        HELPERS
      **************************************************
    */

    // FECHAS

      public function get_next_week() {
        $today = new Carbon();
        $week = [];

        for($i=0; $i<=7; $i++) :
          $to_clone = clone($today);
          $day = $to_clone->addDay($i);
          $week[$i] = (object)[
            'fecha' => $day->format('d-m-Y'),
            'dia_semana' => $day->dayOfWeekIso
          ];
        endfor;

        return $week;
      }

      public function get_fecha_sql($fecha) {
        $fecha = new Carbon($fecha);
        return $fecha->format('Y-m-d');
      }

      public function get_fecha_nosql($fecha) {
        $fecha = new Carbon($fecha);
        return $fecha->format('d-m-Y');
      }

      public function get_fecha_larga($fecha) {
        setlocale(LC_ALL, 'es_ES');
        $fecha = new Carbon($fecha);
        $date = $fecha->formatLocalized('%A, %d de %B del %Y');
        return $date;
      }

      public function user_have_turno($turno) {
        $reserva = Reserva::UserBookTurno(Auth::user()->id, $turno->id, $turno->instalacion->id, $turno->get_fecha_sql())->get();
        if(count($reserva) > 0) :
          return true;
        endif;
        return false;
      }

      public function turno_full_booked($turno) {
        $reserva = Reserva::BookTurno($turno->id, $turno->instalacion->id, $turno->get_fecha_sql())->get();
        if(count($reserva) >= $turno->instalacion->plazas_max) :
          return true;
        endif;
        return false;
      }

      public function sub_full_booked($turno, $sub) {
        $reserva = Reserva::BookSubdivision($turno->id, $sub->id, $turno->get_fecha_sql())->get();
        if(count($reserva) >= $sub->plazas_max) :
          return true;
        endif;
        return false;
      }

      public function sub_get_bookings_number($turno, $sub) {
        $reserva = Reserva::BookSubdivision($turno->id, $sub->id, $turno->get_fecha_sql())->get();
        return $sub->plazas_max-count($reserva);
      }

}
