<?php

namespace App\Http\Controllers;

use Illuminate\Foundation\Auth\Access\AuthorizesRequests;
use Illuminate\Foundation\Bus\DispatchesJobs;
use Illuminate\Foundation\Validation\ValidatesRequests;
use Illuminate\Routing\Controller as BaseController;


// Mains
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Gate;

// Controladores
// use App\Http\Controllers\WSController;
use App\Http\Controllers\HelpersAdminController;

// Utils
use Illuminate\Support\Facades\DB;
use DateTime;
use DateInterval;
use DatePeriod;
use PDF;
use Carbon\Carbon;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Arr;
use Jenssegers\Agent\Agent;
use Illuminate\Support\Facades\Session;

// Models
// use App\Models\NombreModelo;
use App\Models\Configuracion;
use App\Models\CategoriaNoticias; 
use App\Models\Noticia; 
use App\Models\CategoriaAsociado; 
use App\Models\Asociado;
use App\Models\Role;
use App\Models\User;
use App\Models\Slider;


class ViewsAdminController extends BaseController
{

  /*
    **************************************************
      Importacion de controladores
    **************************************************
  */

  // protected $NameController;
  //
  private $helpers_controller;

  public function __construct()
  {
      $this->helpers_controller = new HelpersAdminController; 
      Session::put('agent', $this->helpers_controller->get_agent());
  }


  public function get_home_page() {
    
    // Session::put('agent', HelpersAdminController::get_agent());
    if(Auth::check()) :
      return $this->get_section_website();
    else :
      return view('admin.sections.website', [
      ]);

    endif;

  }


  /*
    **************************************************
      SECCIONES
    **************************************************
  */


  
  public function get_section_website() {
    
    // Session::put('agent', HelpersAdminController::get_agent());
    $sliders = Slider::all();
    $noticias_destacadas = Noticia::Destacadas()->get();

    if(Auth::check()) :
      return view('admin.sections.website', [
        'titulo' => 'Configuraciones de la web',
        'subtitulo' => 'Configuración de todo el contenido de la web',
        'sliders' => $sliders,
        "noticias" => $noticias_destacadas
        ]);
    else :
      return view('admin.sections.website', [
      ]);

    endif;

  }

  public function get_cat_noticias() {
    if(Auth::check()) :
      $view = 'admin.sections.categorias-noticias';

      return view($view, [
          'titulo' => 'Categorias del Blog',
          'subtitulo' => 'Grupos de noticias existentes',
          'categorias' => CategoriaNoticias::all(),
          'agent' => session('agent')
        ]);

    else :
      // return dd(DataController::getNoticiasPortada());
      return view('login', [

      ]);

    endif;
  }

  public function get_categoria(Request $request) {
    // dd(HelpersAdminController::get_agent());
    // dd(count(SubdivisionPrecisa::ByIns($id)->get()));
    $id = $request['id'];
    $vista = Configuracion::find(1)->get_vista('instalaciones');

    switch($vista):
    case 'simple';
      $tipo_vista = 'simple';
      break;
    case 'default';
      $tipo_vista = 'square';
      break;
    endswitch;

    if(Auth::check()) :
      return view('admin.sections.categoria', [
          'categoria' => CategoriaInstalaciones::find($id),
          'instalaciones' => Instalacion::ByCat($id)->get(),
          'vista' => $vista,
          'agent' => $this->helpers_controller->get_agent()
        ]);
    else :
      // return dd(DataController::getNoticiasPortada());
      return view('login', [

      ]);

    endif;

  }

  public function get_usuarios(Request $request) {

    $view = 'admin.sections.usuarios';

    if(Auth::check() && (Gate::any(['is-admin', 'is-root']))) :
      $usuarios = User::ForAdmins();
    elseif(Gate::any(['is-gerente'])) :
      $usuarios = User::ForGerente();
    elseif(Gate::any(['is-supervisor'])) :
      $usuarios = User::ForSupervisor();
    else :
      return view('login', [

      ]);
    endif;

      return view($view, [
        'titulo' => 'Usuarios del sistema',
        'subtitulo' => 'Usuarios',
        'usuarios' => $usuarios->orderBy('name', 'ASC')->get()
      ]);
  }

  public function get_asociados(Request $request) {

    $view = 'admin.sections.asociados';
    $asociados = Asociado::all();

      return view($view, [
        'titulo' => 'Asociados registrados ('.count($asociados).')',
        'subtitulo' => 'Listado de asociados registrados en nuestra base de datos actual',
        'asociados' => $asociados,
        'categorias' => CategoriaAsociado::all()
      ]);
  }

  public function get_cat_asociados() {
    if(Auth::check()) :
      $view = 'admin.sections.categorias-asociados';

      return view($view, [
          'titulo' => 'Categorias de Asociados',
          'subtitulo' => 'Grupos de Asociados existentes',
          'categorias' => CategoriaAsociado::all(),
          'agent' => session('agent')
        ]);

    else :
      // return dd(DataController::getNoticiasPortada());
      return view('login', [
      ]);

    endif;
  }

  public function get_usuario(Request $request) {

    $view = 'admin.sections.usuario-edit';

    if(Auth::check() && (Gate::any(['is-admin', 'is-root']))) :
      $usuarios = User::ForAdmins();
    elseif(Gate::any(['is-gerente'])) :
      $usuarios = User::ForGerente();
    elseif(Gate::any(['is-supervisor'])) :
      $usuarios = User::ForSupervisor();
    else :
      return view('login', [

      ]);
    endif;

      return view($view, [
        'titulo' => 'Usuarios del sistema',
        'subtitulo' => 'Usuarios',
        'usuarios' => $usuarios->orderBy('name', 'ASC')->get()
      ]);
  }

  public function get_reservas_by_user(Request $request) {

    $view = 'admin.sections.usuario-edit';

    if(Auth::check() && (Gate::any(['is-admin', 'is-root']))) :
      $usuarios = User::ForAdmins();
    elseif(Gate::any(['is-gerente'])) :
      $usuarios = User::ForGerente();
    elseif(Gate::any(['is-supervisor'])) :
      $usuarios = User::ForSupervisor();
    else :
      return view('login', [

      ]);
    endif;

      return view($view, [
        'titulo' => 'Usuarios del sistema',
        'subtitulo' => 'Usuarios',
        'usuarios' => $usuarios->orderBy('name', 'ASC')->get()
      ]);
  }


  /*
    **************************************************
      MODALES
    **************************************************
  */


  public function get_modal_new_user(Request $request) {
    $view = 'admin.modals.new-edit-user';
    // $data = $request['data'];
    $roles = Role::Showable()->get();
    $select_roles = [];

    foreach ($roles as $role):
      $insert = (object) ['value' => $role->id, 'texto' => $role->nombre];
      array_push($select_roles, $insert);
    endforeach;
    
    return view($view, [
      "titulo"    => 'Agregar usuario',
      "subtitulo" => 'Creación de un nuevo usuario para el sistema de Asecohe.com',
      "roles" => $select_roles
    ]);
  }
  
  public function get_modal_edit_user(Request $request) {
    $view = 'admin.modals.new-edit-user';
    $data = $request['data'];

    $user = User::findOrFail($data['user_id']);
    $roles = Role::Showable()->get();
    
    $select_roles = [];
    foreach ($roles as $role):
      $insert = (object) ['value' => $role->id, 'texto' => $role->nombre];
      array_push($select_roles, $insert);
    endforeach;
    
    return view($view, [
      "titulo"    => 'Editar usuario',
      "subtitulo" => 'Edición de un nuevo usuario para el sistema de Asecohe.com',
      "usuario" => $user,
      "roles" => $select_roles
    ]);
  }

  public function get_modal_new_asociado()
  {
    $view = 'admin.modals.new-edit-asociado';
    $categorias = CategoriaAsociado::Actives()->get();
    $select_categorias = [];

    foreach ($categorias as $categoria):
      $insert = (object) ['value' => $categoria->id, 'texto' => $categoria->nombre];
      array_push($select_categorias, $insert);
    endforeach;

    return view($view, [
      "titulo" => 'Agregar Asociado',
      "subtitulo" => 'Creación de un nuevo asociado para el sistema de Asecohe.com',
      "categorias" => $categorias,
      "select_categorias" => $select_categorias
    ]);
  }
  
  public function get_modal_edit_asociado(Request $request)
  {
    $view = 'admin.modals.new-edit-asociado';
    $data = $request['data'];
    $categorias = CategoriaAsociado::Actives()->get();
    $asociado = Asociado::where('id', '=', $data['asociado_id'])->first();
    $select_categorias = [];
    
    foreach ($categorias as $categoria):
      $insert = (object) ['value' => $categoria->id, 'texto' => $categoria->nombre];
      array_push($select_categorias, $insert);
    endforeach;

    return view($view, [
      "titulo" => 'Agregar Asociado',
      "subtitulo" => 'Creación de un nuevo asociado para el sistema de Asecohe.com',
      "categorias" => $categorias,
      "asociado" => $asociado,
      "select_categorias" => $select_categorias,
      "selected_categorias" => json_decode($asociado->id_categorias)
    ]);
  }
  
  public function get_modal_new_categoria_asociado(Request $request)
  {
    $view = 'admin.modals.new-edit-categoria-asociado';
    $categorias = CategoriaAsociado::Actives()->get();
    $select_categorias = [];

    foreach ($categorias as $categoria):
      $insert = (object) ['value' => $categoria->id, 'texto' => $categoria->nombre];
      array_push($select_categorias, $insert);
    endforeach;

    return view($view, [
      "titulo" => '',
      "subtitulo" => 'Creación de una nueva categoría de asociado para el sistema de Asecohe.com',
      "categorias" => $categorias,
      "select_categorias" => $select_categorias,
      "is_second" => $request['data']['is_second']
    ]);
  }

  public function get_modal_new_frame()
  {
    $view = 'admin.modals.new-edit-frame';


    return view($view, [
      "titulo" => 'Agregar Frame',
      "subtitulo" => 'Creación de un nuevo frame para el slider de Asecohe.com',
    ]);
  }
  
  public function get_modal_edit_frame(Request $request)
  {
    $view = 'admin.modals.new-edit-frame';
    $data = $request['data'];
    $frame = Slider::where('id', '=', $data['frame_id'])->first();

    return view($view, [
      "titulo" => 'Editar Frame del slider',
      "subtitulo" => 'Edición de frame para el sistema de Asecohe.com',
      "frame" => $frame
    ]);
  }

  public function get_test(Request $request)
  {
    $categoria = CategoriaAsociado::Actives()->where('id', '=', $request['id'])->first();    
    dd($categoria->onDelete());
  }

  /*
    **************************************************
      Funciones Commands
    **************************************************
  */
  

  /*
    **************************************************
      Funciones HELPERS
    **************************************************
  */



}
