<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;

use App\Models\User;
use App\Models\CategoriaAsociado;
use App\Traits\ImageTrait;
use File;

class Asociado extends Model
{
  use HasFactory;

  //  Table
  protected $table = 'asociados';
  // Routes
  protected $logo_route = '/imagenes/asociados/logos/';
  protected $logo_route_admin = '/storage/imagenes/asociados/logos/';

  // Primary ID
  // protected $primaryKey = '';
  
  public function categoria_ppal()
  {
    if (!empty($this->id_categoria_ppal))
      return CategoriaAsociado::where('id', $this->id_categoria_ppal)->first();
      
    return false;
  }
  
  public function categorias()
  {
    $categorias_array = json_decode($this->id_categorias);
    if (count($categorias_array) > 0)
      return CategoriaAsociado::whereIn('id', $categorias_array)->get();
    return false;
  }

  public function delete_categoria($id_cat)
  {
    $cats = json_decode($this->id_categorias);
    if (($key = array_search($id_cat, $cats)) !== false) {
      unset($cats[$key]);
    }
    $this->id_categorias = json_encode($cats);
    return $this->save();
  }

  public function toggle_destacar()
  {
    $this->destacado = !$this->destacado;
    $this->save();
  }

  public function is_destacado() {
    return (bool) $this->destacado;
  }

  public function is_activo() {
    return (bool) $this->activo;
  }

  // GETTERS
  public function get_estado()
  {
    $estatus = $this->is_activo();
    $estado = (object) [];
    
    switch ($estatus):
      case 1:
        $estado->nombre = "Activo";
        $estado->bg_color = 'rgb(174, 209, 158)';
        $estado->color = 'rgb(73, 73, 73)';
        break;
      case 0:
        $estado->nombre = "Inactivo";
        $estado->bg_color = 'rgb(255, 0, 0)';
        $estado->color = 'rgb(73, 73, 73)';
        break;
    endswitch;

    return $estado;
  }

  public function get_categorias_array()
  {
    return json_decode($this->id_categorias);
  }

  // IMAGENES 
  public function get_dir_imagen() {
    return $this->logo_route;
  }  
  public function get_dir_admin_imagen() {
    return $this->logo_route_admin;
  }  
  public function get_imagen() {
    return $this->get_dir_imagen().$this->imagen;
  }
  public function set_imagen($file_name) {
    return $this->imagen = $file_name;
  }
  public function get_imagen_admin() {
    return $this->get_dir_admin_imagen().$this->imagen;
  }
  public function del_imagen() {
    if(count($this::ByImg()->get()) <= 1) :
      if(File::exists(public_path($this->get_imagen_admin()))){
          File::delete(public_path($this->get_imagen_admin()));
          return true;
      }else{
          dd('File does not exists.');
      }
    else :
      return false;
    endif;
  }

  //SCOPE
  public function scopeByImg($query) {
    return $query->where('imagen', '=', $this->imagen);
  }
  
  /*
  ********************
  SCOPES
  ********************
  */

  public function scopeDestacados($query)
  {
    return $query->where('destacado', '=', 1)->limit(4);
  }

  public function scopeActives($query)
  {
    return $query->where('activo', '=', 1);
  }

}