<?php

namespace App\Models;

use Illuminate\Contracts\Auth\MustVerifyEmail;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Foundation\Auth\User as Authenticatable;
use Illuminate\Notifications\Notifiable;
use Laravel\Fortify\TwoFactorAuthenticatable;
use Laravel\Jetstream\HasProfilePhoto;
use Laravel\Sanctum\HasApiTokens;

use App\Models\UserInstalaciones;
use App\Models\Role;
use App\Models\Reserva;

class User extends Authenticatable
{
    use HasApiTokens;
    use HasFactory;
    use HasProfilePhoto;
    use Notifiable;
    use TwoFactorAuthenticatable;

    /**
     * The attributes that are mass assignable.
     *
     * @var string[]
     */
    protected $fillable = [
        'name',
        'email',
        'telefono',
        'localidad',
        'direccion',
        'password',
        'status',
    ];

    /**
     * The attributes that should be hidden for serialization.
     *
     * @var array
     */
    protected $hidden = [
        'password',
        'remember_token',
        'two_factor_recovery_codes',
        'two_factor_secret',
    ];

    /**
     * The attributes that should be cast.
     *
     * @var array
     */
    protected $casts = [
        'email_verified_at' => 'datetime',
    ];

    /**
     * The accessors to append to the model's array form.
     *
     * @var array
     */
    protected $appends = [
        'profile_photo_url',
    ];

    // ****************************************
    // RELATIONS
    public function role() {
      return $this->belongsTo('App\Models\Role', 'id_role');
    }

    public function status() {
      return $this->belongsTo('App\Models\UserStatus', 'id_status');
    }

    public function instalaciones() {
      // return $this->hasManyThrough('App\Models\Instalacion', 'App\Models\UserInstalaciones');
      return UserInstalaciones::ForUser($this->id)->get();
    }

    // GETTERS
    public function get_estado() {

      $estatus = $this->status->nombre;

      $estado = (object)[];
      $estado->nombre = $estatus;

      switch($estatus) :
        case 'activo':
          $estado->bg_color = 'rgb(174, 209, 158)';
          $estado->color = 'rgb(73, 73, 73)';
          break;
        case 'pausado':
          $estado->bg_color = 'rgb(209, 159, 158)';
          $estado->color = 'rgb(73, 73, 73)';
          break;
        case 'bloqueado':
          $estado->bg_color = 'rgb(210, 158, 80)';
          $estado->color = 'rgb(73, 73, 73)';
          break;
        case 'eliminado':
          $estado->bg_color = 'rgb(255, 0, 0)';
          $estado->color = 'rgb(73, 73, 73)';
          break;
      endswitch;

      return $estado;
    }

    public function get_ultima_reserva() {
      $reserva = Reserva::where('id_user', '=', $this->id)
      ->OrderBy('fecha', 'DESC')
      ->OrderBy('hora_inicio', 'DESC')
      ->limit(1)
      ->first();

      return $reserva;
    }

    // ****************************************
    // SCOPES

    public function scopeForSelect($query) {
      return $query->select('id', 'name', 'telefono', 'email');
    }

    public function scopeForAdmins($query) {
      // VEREMOS GERENTE (2), SUPERVISOR (3) y usuario normal (4)
      $roles=[
        2,3,4
      ];
      return $query->whereIn('id_role', $roles);

    }
    public function scopeForGerente($query) {
      // VEREMOS SUPERVISOR (3) y usuario normal (4)
      $roles=[
        3,4
      ];
      return $query->whereIn('id_role', $roles);
    }
    public function scopeForSupervisor($query) {
      $roles=[
        4
      ];
      return $query->whereIn('id_role', $roles);
    }
}
