<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;

use Carbon\Carbon;
use File;

use App\Models\ConfigInstalacion;
use App\Models\Configuracion;

class Instalacion extends Model
{
    use HasFactory;

    //  Table
    protected $table = 'instalaciones';

    // Primary ID
    protected $primaryKey = 'id';


    public function __construct() {
      // $this->agent = new Agent();
      // $this->get_imagen = '/storage/imagenes/instalaciones/'.$this->imagen;
    }

    /*
    ********************
      RELATIONS
    ********************
    */

    // public function config() {
    //   return $this->hasOne('App\Models\ConfigInstalacion', 'IDCLI', 'IDCLI');
    //   // return $this->hasOne('App\Models\ClienteConfig', 'COD', 'CODCLI');
    // }

    public function turnos_def() {
        return $this->hasMany('App\Models\TurnoDefault', 'id_ins');
    }

    public function turnos_precisos() {
        return $this->hasMany('App\Models\TurnoPreciso', 'id_ins');
    }

    public function subdivisiones_def() {
        return $this->hasMany('App\Models\SubdivisionDefault', 'id_ins');
    }

    public function subdivisiones_pre() {
        return $this->hasMany('App\Models\SubdivisionPrecisa', 'id_ins');
    }


    /*
    ********************
      HELPERS
    ********************
    */

    public function has_subdivisiones() {
      return ((count($this->subdivisiones_def) > 0) || (count($this->subdivisiones_pre) > 0));
    }

    public function has_subdivisiones_pre() {
      return ((count($this->subdivisiones_pre) > 0));
    }

    public function get_apertura_datetime() {
      $now = new Carbon();
      if($this->h_apertura_reserva > 0) :
        $end_datetime = $now->addHours($this->h_apertura_reserva);
      else :
        $end_datetime = $now->addHours(Configuracion::find('1')->get_time_apertura_reservas());
      endif;
      return $end_datetime;
    }

    public function isActive() {
      return $this->activo == 1;
    }

    public function get_imagen() {
      return '/storage/imagenes/instalaciones/'.$this->imagen;
    }

    public function get_public_imagen() {
      return '/public/imagenes/instalaciones/'.$this->imagen;
    }
    public function get_dir_imagen() {
      return '/storage/imagenes/instalaciones/';
    }

    public function get_dir_docs() {
      return '/storage/docs/instalaciones/'.$this->archivo_normativa;
    }

    public function get_public_dir_docs() {
      return '/public/docs/instalaciones/'.$this->archivo_normativa;
    }

    public function del_imagen() {
      // dd(Instalacion::ByImg($this->imagen)->get());
      if(count(Instalacion::ByImg()->get()) <= 1) :
        if(File::exists(public_path($this->get_imagen()))){
            File::delete(public_path($this->get_imagen()));
            return true;
        }else{
            dd('File does not exists.');
        }
      else :
        return false;
      endif;
    }

    public function del_normativa() {
      if(count(Instalacion::ByNormativa()->get()) <= 1) :
        if(File::exists(public_path($this->get_dir_docs()))){
            File::delete(public_path($this->get_dir_docs()));
            return true;
        }else{
            dd('File does not exists.');
        }
      else :
        return false;
      endif;
    }

    public function get_hora_apertura() {
      $hora = new Carbon($this->hora_apertura);
      $hora = $hora->format('H:i');
      return $hora;
    }
    public function get_hora_cierre() {
      $hora = new Carbon($this->hora_cierre);
      $hora = $hora->format('H:i');
      return $hora;
    }

    public function categoria() {
      if($this->id_categoria != 0) :
        return $this->hasOne('App\Models\CategoriaInstalaciones', 'id', 'id_categoria');
      else :
        return null;
      endif;
    }

    public function get_next_week() {
      dd($this->turnos_def);
    }


    /*
        ********************
          SCOPES
        ********************
    */

    // public function scopeForAgencia($query, $agencia_id) {
    //   return $query->where('CODAGE', '=', $agencia_id);
    // }

    public function scopeByCat($query, $id) {
      return $query->where('id_categoria', '=', $id);
    }

    public function scopeSinCategoria($query) {
      return $query->where('id_categoria', '=', 0);
    }

    public function scopeActives($query) {
      return $query->where('activo', '=', 1);
    }

    public function scopeByImg($query) {
      return $query->where('imagen', '=', $this->imagen);
    }

    public function scopeByNormativa($query) {
      return $query->where('archivo_normativa', '=', $this->archivo_normativa);
    }

}
