<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;

use Carbon\Carbon;

use App\Models\User;
use App\Models\ReservaStatus;

class Reserva extends Model
{
    use HasFactory;

    //  Table
    protected $table = 'reservas';

    // Primary ID
    // protected $primaryKey = '';

    // GETTERS
    public function status() {
      return $this->belongsTo(ReservaStatus::class, 'id_status');
    }
    public function get_status() {
      return ReservaStatus::find($this->id_status);
    }

    public function user() {
      return $this->belongsTo(User::class, 'id_user');
    }

    public function instalacion() {
      return $this->belongsTo('App\Models\Instalacion', 'id_ins');
    }

    public function division() {
      if(!$this->tipo_sub) :
        return $this->belongsTo('App\Models\SubdivisionDefault', 'id_sub');
      else :
        return $this->belongsTo('App\Models\SubdivisionPrecisa', 'id_sub');
      endif;
    }

    public function tipo_division() {
        return $this->belongsTo('App\Models\TipoSub', 'id_tipo_sub');
    }

    public function turno() {
      if(!$this->tipo_turno) :
        return $this->belongsTo('App\Models\TurnoDefault', 'id_turno');
      else :
        return $this->belongsTo('App\Models\TurnoPreciso', 'id_turno');
      endif;
    }

    public function get_time_booking() {
      $hora = new Carbon($this->created_at);
      $hora = $hora->format('H:i');
      return $hora;
    }

    public function get_fecha_booking() {
      $fecha = new Carbon($this->created_at);
      $fecha = $fecha->format('d-m-Y');
      return $fecha;
    }

    public function get_fecha_hora() {
      $fecha = new Carbon($this->fecha);
      $fecha = $fecha->format('d-m-Y');
      $hora_ini = new Carbon($this->hora_inicio);
      $hora_ini = $hora_ini->format('H:i');
      $hora_fin = new Carbon($this->hora_final);
      $hora_fin = $hora_fin->format('H:i');

      return $fecha.' '.$hora_ini.'-'.$hora_fin;
    }

    public function get_estado() {

      $estatus = $this->get_status()->nombre;

      $estado = (object)[];
      $estado->nombre = $estatus;

      switch($estatus) :
        case 'activo':
          $estado->bg_color = 'rgb(174, 209, 158)';
          $estado->color = 'rgb(73, 73, 73)';
          $estado->icon = '<i class="fas fa-check"></i>';
          break;
        case 'cancelada':
          $estado->bg_color = 'rgb(209, 159, 158)';
          $estado->color = 'rgb(73, 73, 73)';
          $estado->icon = '<i class="fas fa-times"></i>';
          break;
        case 'baneada':
          $estado->bg_color = 'rgb(210, 158, 80)';
          $estado->color = 'rgb(73, 73, 73)';
          $estado->icon = '<i class="fas fa-ban"></i>';
          break;
        case 'pasada':
          $estado->bg_color = 'rgb(45, 45, 45)';
          $estado->color = 'rgb(240, 240, 240)';
          $estado->icon = '<i class="fas fa-vote-yea"></i>';
          break;
        case 'en curso':
          $estado->bg_color = 'rgb(52, 120, 163)';
          $estado->color = 'rgb(240, 240, 240)';
          $estado->icon = '<i class="fas fa-smile"></i>';
          break;
        case 'no presentado':
          $estado->bg_color = 'rgb(205, 21, 17)';
          $estado->color = 'rgb(240, 240, 240)';
          $estado->icon = '<i class="fas fa-frown"></i>';
          break;
      endswitch;

      return $estado;
    }

    public function change_estado() {
      if($this->activo) :
        $this->activo = 0;
      else :
        $this->activo = 1;
      endif;

      // return $this->get_estado();
    }

    // CHANGE OF STATUSES
    public function cancelar_reserva() {
      $this->id_status = 2;
      $this->save();
    }

    public function banear_reserva() {
      $this->id_status = 3;
      $this->save();
    }

    public function finalizar_reserva() {
      $this->id_status = 4;
      $this->save();
    }

    public function no_presentado_reserva() {
      $this->id_status = 6;
      $this->save();
    }

    /*
        ********************
          SCOPES
        ********************
    */

    public function scopeUserBookTurno($query, $user_id, $turno_id, $ins_id, $date) {
      return $query->where('id_user', '=', $user_id)
      ->where('id_ins', '=', $ins_id)
      ->where('id_turno', '=', $turno_id)
      ->where('fecha', '=', $date);
    }

    public function scopeBookTurno($query, $turno_id, $ins_id, $date) {
      return $query->where('id_ins', '=', $ins_id)
      ->where('id_turno', '=', $turno_id)
      ->where('fecha', '=', $date);
    }

    public function scopeBookSubdivision($query, $turno_id, $sub_id, $date) {
      return $query->where('id_turno', '=', $turno_id)
      ->where('id_sub', '=', $sub_id)
      ->where('fecha', '=', $date);
    }

    public function scopeHoy($query) {
      $from = Carbon::today();
      $to = Carbon::today();
      return $query->whereBetween('fecha', [$from, $to])->orderBy('fecha', 'DESC');
    }

    public function scopeSemana($query) {
      $from = Carbon::today();
      $to = Carbon::today()->addDays(7);
      return $query->whereBetween('fecha', [$from, $to])->orderBy('fecha', 'DESC');
    }

    public function scopeMes($query) {
      $from = Carbon::today();
      $to = Carbon::today()->addMonth(1);
      return $query->whereBetween('fecha', [$from, $to])->orderBy('fecha', 'DESC');
    }

    public function scopeAllByDate($query) {
      return $query->orderBy('fecha', 'DESC')->orderBy('hora_inicio')->limit(300);
    }

    public function scopeForAdmins($query) {
      return $query->orderBy('fecha', 'DESC')->orderBy('hora_inicio')->limit(300);
    }

    public function scopeForInstalaciones($query, $ins) {
      return $query->whereIn('id_ins', $ins);
    }

    public function scopeForUser($query, $user_id) {
      return $query->where('id_user', '=', $user_id);
    }

    public function scopeActivas($query) {
      return $query->whereHas('status', function($q) {
        $q->where('nombre', 'activo');
      });
    }

}
