<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;

use Carbon\Carbon;

use App\Http\Controllers\HelpersController;

use App\Models\User;

class SubdivisionDefault extends Model
{
    use HasFactory;

    //  Table
    protected $table = 'subdivisiones_default';

    // Primary ID
    // protected $primaryKey = '';


    /*
    ********************
    RELATIONS
    ********************
    */

    public function tipo() {
      return $this->belongsTo('App\Models\TipoSub', 'id_tipo');
    }


    public function instalacion() {
      return $this->belongsTo('App\Models\Instalacion', 'id_ins');
    }

    public function type() {
      return 0;
    }

    /*
        ********************
          HELPERS
        ********************
    */


    public function is_open_for_turno($turno) {
      $apertura = new Carbon($this->hora_apertura);
      $cierre = new Carbon($this->hora_cierre);

      $turno_apertura = new Carbon($turno->hora_comienzo);

      if ($turno_apertura->between($apertura, $cierre)) {
        return true;
      }

      return false;
    }


    public function is_full_booked($turno) {
      return HelpersController::sub_full_booked($turno, $this);
    }

    public function free_slots($turno) {
      return HelpersController::sub_get_bookings_number($turno, $this);
    }

    public function get_estado() {
      $estado = (object)[];

      if($this->activo) :
        $estado->color = 'rgb(174, 209, 158)';
        $estado->nombre = 'activo';
      else :
        $estado->color = 'rgb(209, 159, 158)';
        $estado->nombre = 'inactivo';
      endif;

      return $estado;
    }

    public function change_estado() {
      if($this->activo) :
        $this->activo = 0;
      else :
        $this->activo = 1;
      endif;

      // return $this->get_estado();
    }

    /*
        ********************
          SCOPES
        ********************
    */

    // OBTENER SUBDIVISIONES PARA UN TURNO.
    public function scopeForDiaSemana($query, $turno) {
      return $query->where(function () use ($query, $turno) {
        $this->is_open_for_turno($turno);
      })
      ->whereHas('instalacion', function ($q) use ($turno) {
        $q->where('id', $turno->instalacion->id);
      });
    }

    public function scopeForDiaSemanaPreciso($query, $turno) {
      return $query->where(function () use ($query, $turno) {
        $this->is_open_for_turno($turno);
      })
      ->whereHas('instalacion', function ($q) use ($turno) {
        $q->where('id', $turno->instalacion->id);
      });
    }

    public function scopeByIns($query, $ins_id) {
      return $query->where('id_ins', $ins_id);
    }



}
