<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;

use Carbon\Carbon;

use App\Http\Controllers\HelpersController;

use App\Models\User;
use App\Models\Instalacion;
use App\Models\Reserva;


class TurnoDefault extends Model
{
    use HasFactory;

    //  Table
    protected $table = 'turnos_default';

    // Primary ID
    // protected $primaryKey = '';


    // public $FECHA = $this->get_fecha();
    public $TIPO = "default";

    // public function __construct() {
    //   // $this->get_next_week();
    //   dd($this);
    //
    // }

    public function dia_semana() {
      $dias = [
        'lunes', 'martes', 'miercoles', 'jueves', 'viernes', 'sábado', 'domingo'
      ];

      return $dias[$this->dia_semana-1];
    }

    public function get_hora_comienzo() {
      $hora = new Carbon($this->hora_comienzo);
      $hora = $hora->format('H:i');
      return $hora;
    }

    public function get_hora_termino() {
      $hora = new Carbon($this->hora_termino);
      $hora = $hora->format('H:i');
      return $hora;
    }

    public function instalacion() {
      return $this->belongsTo(Instalacion::class, 'id_ins');
    }

    public function type() {
      return 0;
    }

    // Cada envio tiene un unico usuario que ha creado el envio
    // public function usuario()
    // {
    //     return $this->belongsTo('App\Models\Usuario', 'IDUSU');
    // }


    /*
        ********************
          HELPERS
        ********************
    */

    public function get_fecha() {
      $next_week = HelpersController::get_next_week();
      foreach($next_week as $dia) :
        if($dia->dia_semana == $this->dia_semana) :
          return $dia;
        endif;
      endforeach;
    }

    public function get_fecha_of() {
      return HelpersController::get_fecha_nosql($this->get_fecha()->fecha);
    }

    public function get_fecha_of_larga() {
      return HelpersController::get_fecha_larga($this->get_fecha()->fecha);
    }

    public function get_fecha_sql() {
      return HelpersController::get_fecha_sql($this->get_fecha()->fecha);
    }

    private function get_instalacion($id = 1) {
      // dd($this);
      $ins = Instalacion::where('id', '=', (int)$id)->get();
      return $ins[0];
    }

    private function get_comienzo_datetime($id_ins, $fecha_turno) {
      $fecha = new Carbon($fecha_turno);
      $datetime = $fecha->format('Y-m-d').' '.$this->hora_comienzo;
      $back = new Carbon($datetime);
      return $back;
    }

    public function compare_times($id_ins, $fecha_dia) {
      $now = new Carbon();
      $turno_datetime = $this->get_comienzo_datetime($id_ins, $fecha_dia);
      $ins_limit_time = $this->get_instalacion($id_ins)->get_apertura_datetime();

      if ($turno_datetime->between($now, $ins_limit_time)) {
        return true;
      }

      return false;
    }

    public function isin_next_week($id_ins, $fecha_dia) {
      $now = new Carbon();
      $turno_datetime = $this->get_comienzo_datetime($id_ins, $fecha_dia);
      $next_week = $now->clone();
      $ins_limit_time = $next_week->addDays(7);

      if ($turno_datetime->between($now, $ins_limit_time)) {
        return true;
      }

      return false;
    }

    public function is_booked() {
      return HelpersController::user_have_turno($this);
    }

    public function is_full_booked() {
      return HelpersController::turno_full_booked($this);
    }

    /*
        ********************
          SCOPES
        ********************
    */


    public function scopeByIns($query, $id_ins) {
      return $query->where([['id_ins', '=', $id_ins]]);
    }

    public function scopeByInsDay($query, $id_ins, $dia) {
      return $query->where([['id_ins', '=', $id_ins], ['dia_Semana', '=', $dia]]);
    }

    public function scopeBetweenDates($query, $from, $to) {
      return $query->whereBetween('fecha', [$from, $to]);
    }

}
