<?php

namespace App\Providers;

use App\Actions\Fortify\CreateNewUser;
use App\Actions\Fortify\ResetUserPassword;
use App\Actions\Fortify\UpdateUserPassword;
use App\Actions\Fortify\UpdateUserProfileInformation;
use Illuminate\Cache\RateLimiting\Limit;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\RateLimiter;
use Illuminate\Support\ServiceProvider;
use Laravel\Fortify\Fortify;

use App\Models\User;
use Illuminate\Support\Facades\Hash;
use Illuminate\Validation\ValidationException;
use Illuminate\Support\Facades\DB;


class FortifyServiceProvider extends ServiceProvider
{
    /**
     * Register any application services.
     *
     * @return void
     */
    public function register()
    {
        //
    }

    /**
     * Bootstrap any application services.
     *
     * @return void
     */
    public function boot()
    {
        Fortify::createUsersUsing(CreateNewUser::class);
        Fortify::updateUserProfileInformationUsing(UpdateUserProfileInformation::class);
        Fortify::updateUserPasswordsUsing(UpdateUserPassword::class);
        Fortify::resetUserPasswordsUsing(ResetUserPassword::class);

        RateLimiter::for('login', function (Request $request) {
            $email = (string) $request->email;
            return Limit::perMinute(5)->by($email.$request->ip());
        });

        RateLimiter::for('two-factor', function (Request $request) {
            return Limit::perMinute(5)->by($request->session()->get('login.id'));
        });


        ## If we need to redirect after login We will use this
        // $this->app->singleton(LoginResponseContract::class, LoginResponse::class);

        Fortify::authenticateUsing(
          function (Request $request) {

            // Recogemos el user de nuestro sistema
            $user = User::where('email', '=', $request->email)->first();
            if($user == null) :
              $error = "El usuario con el que intenta acceder no existe.";
              throw ValidationException::withMessages([
                "Login incorrecto: ".$error
              ]);
            endif;

            // status del usuario es activo
            if($user->status->nombre == 'activo') :

                // El usuario tiene que cambiar su contraseña.
                if($user->cambiar_password) :
                    // confirm password is not empty
                    if($request['conf_password'] != '') :

                        // password and confirm password are same?
                        if($request['conf_password'] == $request['password']) :
                            $user->password = Hash::make($request['password']);
                            $user->cambiar_password = 0;
                            $user->save();

                            return $user;
                            // password and confirmation doesn't match
                        else :
                            $error = "La contraseña y su confirmación deben coincidir.";
                            throw ValidationException::withMessages([
                              "Login incorrecto: ".$error
                            ]);
                        endif;

                    // Confirmation is empty
                    else :
                        $error = "Su usuario requiere crear nueva contraseña. La confirmación debe rellenarse.";
                        throw ValidationException::withMessages([
                          $error
                        ]);
                    endif;
                  else :
                      if($request['password'] == '') :
                        $error = "password";
                        throw ValidationException::withMessages([
                          $error
                        ]);
                      else :
                        if ($user && Hash::check($request->password, $user->password)) :
                            return $user;
                        else :
                          $back = (object)[
                            'message' => 'La contraseña proporcionada no corresponde con la que tenemos almacenada en nuestra base de datos.',
                            'title' => 'Contraseña incorrecta',
                            'type' => 'error'
                          ];
                          $back = json_encode($back);
                          throw ValidationException::withMessages([
                            'message' => $back
                          ]);
                        endif;
                      endif;
                  endif;


            // status del usuario NO es activo
            else :
                $back = (object)[
                  'message' => 'Este usuario está bloqueado o desactivado en estos momentos. Por favor, pongase en contacto con los administradores.',
                  'title' => 'Usuario sin permiso de acceso',
                  'type' => 'error'
                ];
                $back = json_encode($back);
                throw ValidationException::withMessages([
                  'message' => $back
                ]);

                // CERRAMOS VACTIVO IF
            endif;
        });
    }
}