
@extends('dashboard')


@section('section')

  <div class="admin-container" id="test">

    <div class="admin-main-title">

      <div class="row same-height">
        <div class="col-12 col-md-8 same-height-col">
          <h1>{{ $titulo }}</h1>
          <div class="subtitle">
            <p>{{$subtitulo}}</p>
          </div>
        </div>
        <div class="col-12 col-md-4 same-height-col text-right text-left-mobile">
          @canany(['is-root', 'is-admin', 'is-gerente'])
            <ul class="wrapper">
              <li class="icon view">
                <span class="tooltip">Nueva Reserva</span>
                <span class="is-link" onclick="openNuevaReserva()"> <i class='bx bx bx-calendar-plus' ></i> </span>
              </li>
            </ul>
          @endcanany
          <ul class="wrapper grouped">
            <li class="icon view @if(isset($filtro) && $filtro == 'hoy') active @endif" onclick="go('reservas-hoy')">
              <span class="tooltip">Reservas hoy</span>
              <span class="is-link"> <i class='bx bxs-calendar-event' ></i></span>
            </li>
            <li class="icon view @if(isset($filtro) && $filtro == 'semana') active @endif" onclick="go('reservas-semana')">
              <span class="tooltip">Reservas semana</span>
              <span class="is-link"> <i class='bx bxs-calendar-week' ></i></span>
            </li>
            <li class="icon view @if(isset($filtro) && $filtro == 'mes') active @endif" onclick="go('reservas-mes')">
              <span class="tooltip">Reservas mes</span>
              <span class="is-link"> <i class='bx bxs-calendar' ></i></span>
            </li>
            <li class="icon view @if(!isset($filtro)) active @endif" onclick="go('reservas')">
              <span class="tooltip">Todas las reservas</span>
              <span class="is-link"> <i class='bx bxs-calendar-alt' ></i></span>
            </li>
          </ul>

        </div>
      </div>
    </div>


    {{-- **************************************** --}}
    {{-- LISTADO RESERVAS ************ --}}
    {{-- **************************************** --}}

    <div class="col-md-12">

      <div class="card">
        <div class="card-header">
          <span class="titulo">Listado</span>
          <span class="open-close-card" onclick="toggleCardBody(this, 1)">
            @if(session('pestana_actual') == 1)
              <i class='bx bx2 bx-folder-minus'></i>
            @else
              <i class='bx bx2 bxs-folder-plus'></i>
            @endif
          </span>
        </div>

        <div class="card-body coll-able pest-1 @if(session('pestana_actual') == 1)opened @endif">

          <table id="table-here-nores" class="table table-hover no-footer reservas evenor" role="grid">

              <thead class="thead-dark">
                <tr role="row" class="">
                    <th scope="col" style="width:3%">id</th>
                    <th scope="col" style="width:15%" class="left">Referencia</th>
                    <th scope="col" style="width:15%" class="left">Fecha</th>
                    <th scope="col" style="width:15%" class="left">Usuario</th>
                    <th scope="col" style="width:20%">Instalación</th>
                    <th scope="col" style="width:10%">Estado</th>
                    <th scope="col" style="width:auto" class="left no-click" >Acciones</th>
                </tr>
              </thead>

            <tbody>
              @if(count($reservas) > 0)
                @foreach($reservas as $div)
                  <tr class="div-reservas">

                    <td scope="row" data-label="id">
                      <span class="small-text">{{ $div->id }} </span>
                    </td>
                    <td class="left" data-label="referencia">
                      <span class="small-text">{{ $div->ref }} </span>
                    </td>
                    <td class="left" data-label="fecha">
                      <span class="small-text">{{ $div->get_fecha_hora() }} </span>
                    </td>
                    <td class="left" data-label="usuario">
                      <span class="small-text with-link" onclick="openUsuario({{$div->user->id}})">{{ $div->user->name }} </span>
                    </td>
                    <td class="left" data-label="instalacion">
                      <span class="small-text with-link" onclick="openInstalacion({{$div->instalacion->id}})">{{ $div->instalacion->nombre }} </span>
                    </td>

                    <td class="text-center" data-label="Estado" style="background-color:{{ $div->get_estado()->bg_color }}; @if($div->id_status >3)color:#FFF;@endif" >
                      <span class="small-text text-center">{{ $div->get_estado()->nombre }}</span>
                    </td>

                    <td class="text-right" data-label="Acciones">

                      <button class="btn btn-primary btn-actions" data-toggle="tooltip" data-placement="left" title="Editar"
                      onclick="openReserva({{$div->id}})">
                        <i class='bx bx-calendar-edit' ></i>
                      </button>

                      <button class="btn btn-danger btn-actions eliminar" data-toggle="tooltip" data-placement="top" title="Eliminar del sistema"
                      onclick="eliminarTipoSub({{ $div->id }}, this);" data-name="{{$div->ref}}">
                        <i class='bx bxs-trash'></i>
                      </button>

                    </td>

                  </tr>
                @endforeach

              @else
                <tr><td colspan="7">No hay reservas que mostrar bajo el criterio seleccionado</td></tr>
              @endif

            </tbody>

          </table>

        </div> <!-- CARD BODY -->

        <div class="card-footer">

        </div>
      </div>



    </div>

  </div>
@endsection

<script src="/js/admin/helper.js"></script>
<script src="/js/admin/reserva.js"></script>


<script>

window.onload = function() {
  $('[data-toggle=tooltip]').tooltip();

  //
  // $('.card-body.coll-able').css('display', 'none');
  //
  // @if(session('pestana_actual'))
  //   $('.card-body.coll-able.pest-'+'{{ session('pestana_actual')}}').show('fast');
  // @else
  //   toggleCardBody($('.card-body.coll-able.pest-1'), '1');
  // @endif


  $(".reservas").fancyTable({
      // sortColumn: 2,
      globalSearch: false,
      perPage: 20,
      pagination: true,
      paginationClass : 'btn btn-secundary',
      paginationClassActive : 'btn btn-primary',
      inputPlaceholder: 'Buscar...',
      inputStyle: 'color:rgb(106, 136, 180);font-size:0.8rem;',
      onUpdate: function() {
        // $('.toggleable').hide('fast');
      }
  });

};

</script>
